# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division, print_function, unicode_literals

from ._document import ObjectDocumentModel
from ._paragraph import Paragraph
from ._sentence import Sentence
